/*
Copyright 2021 Rancher Labs, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v3

import (
	"context"
	"time"

	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	v3 "github.com/rancher/rancher/pkg/apis/cluster.cattle.io/v3"
	"github.com/rancher/wrangler/pkg/generic"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type ClusterUserAttributeHandler func(string, *v3.ClusterUserAttribute) (*v3.ClusterUserAttribute, error)

type ClusterUserAttributeController interface {
	generic.ControllerMeta
	ClusterUserAttributeClient

	OnChange(ctx context.Context, name string, sync ClusterUserAttributeHandler)
	OnRemove(ctx context.Context, name string, sync ClusterUserAttributeHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() ClusterUserAttributeCache
}

type ClusterUserAttributeClient interface {
	Create(*v3.ClusterUserAttribute) (*v3.ClusterUserAttribute, error)
	Update(*v3.ClusterUserAttribute) (*v3.ClusterUserAttribute, error)

	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v3.ClusterUserAttribute, error)
	List(namespace string, opts metav1.ListOptions) (*v3.ClusterUserAttributeList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v3.ClusterUserAttribute, err error)
}

type ClusterUserAttributeCache interface {
	Get(namespace, name string) (*v3.ClusterUserAttribute, error)
	List(namespace string, selector labels.Selector) ([]*v3.ClusterUserAttribute, error)

	AddIndexer(indexName string, indexer ClusterUserAttributeIndexer)
	GetByIndex(indexName, key string) ([]*v3.ClusterUserAttribute, error)
}

type ClusterUserAttributeIndexer func(obj *v3.ClusterUserAttribute) ([]string, error)

type clusterUserAttributeController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewClusterUserAttributeController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) ClusterUserAttributeController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &clusterUserAttributeController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromClusterUserAttributeHandlerToHandler(sync ClusterUserAttributeHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v3.ClusterUserAttribute
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v3.ClusterUserAttribute))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *clusterUserAttributeController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v3.ClusterUserAttribute))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateClusterUserAttributeDeepCopyOnChange(client ClusterUserAttributeClient, obj *v3.ClusterUserAttribute, handler func(obj *v3.ClusterUserAttribute) (*v3.ClusterUserAttribute, error)) (*v3.ClusterUserAttribute, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *clusterUserAttributeController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *clusterUserAttributeController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *clusterUserAttributeController) OnChange(ctx context.Context, name string, sync ClusterUserAttributeHandler) {
	c.AddGenericHandler(ctx, name, FromClusterUserAttributeHandlerToHandler(sync))
}

func (c *clusterUserAttributeController) OnRemove(ctx context.Context, name string, sync ClusterUserAttributeHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromClusterUserAttributeHandlerToHandler(sync)))
}

func (c *clusterUserAttributeController) Enqueue(namespace, name string) {
	c.controller.Enqueue(namespace, name)
}

func (c *clusterUserAttributeController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controller.EnqueueAfter(namespace, name, duration)
}

func (c *clusterUserAttributeController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *clusterUserAttributeController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *clusterUserAttributeController) Cache() ClusterUserAttributeCache {
	return &clusterUserAttributeCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *clusterUserAttributeController) Create(obj *v3.ClusterUserAttribute) (*v3.ClusterUserAttribute, error) {
	result := &v3.ClusterUserAttribute{}
	return result, c.client.Create(context.TODO(), obj.Namespace, obj, result, metav1.CreateOptions{})
}

func (c *clusterUserAttributeController) Update(obj *v3.ClusterUserAttribute) (*v3.ClusterUserAttribute, error) {
	result := &v3.ClusterUserAttribute{}
	return result, c.client.Update(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *clusterUserAttributeController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), namespace, name, *options)
}

func (c *clusterUserAttributeController) Get(namespace, name string, options metav1.GetOptions) (*v3.ClusterUserAttribute, error) {
	result := &v3.ClusterUserAttribute{}
	return result, c.client.Get(context.TODO(), namespace, name, result, options)
}

func (c *clusterUserAttributeController) List(namespace string, opts metav1.ListOptions) (*v3.ClusterUserAttributeList, error) {
	result := &v3.ClusterUserAttributeList{}
	return result, c.client.List(context.TODO(), namespace, result, opts)
}

func (c *clusterUserAttributeController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), namespace, opts)
}

func (c *clusterUserAttributeController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (*v3.ClusterUserAttribute, error) {
	result := &v3.ClusterUserAttribute{}
	return result, c.client.Patch(context.TODO(), namespace, name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type clusterUserAttributeCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *clusterUserAttributeCache) Get(namespace, name string) (*v3.ClusterUserAttribute, error) {
	obj, exists, err := c.indexer.GetByKey(namespace + "/" + name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v3.ClusterUserAttribute), nil
}

func (c *clusterUserAttributeCache) List(namespace string, selector labels.Selector) (ret []*v3.ClusterUserAttribute, err error) {

	err = cache.ListAllByNamespace(c.indexer, namespace, selector, func(m interface{}) {
		ret = append(ret, m.(*v3.ClusterUserAttribute))
	})

	return ret, err
}

func (c *clusterUserAttributeCache) AddIndexer(indexName string, indexer ClusterUserAttributeIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v3.ClusterUserAttribute))
		},
	}))
}

func (c *clusterUserAttributeCache) GetByIndex(indexName, key string) (result []*v3.ClusterUserAttribute, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v3.ClusterUserAttribute, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v3.ClusterUserAttribute))
	}
	return result, nil
}
